Long: no-progress-meter
Help: Do not show the progress meter
See-also: verbose silent
Added: 7.67.0
---
Option to switch off the progress meter output without muting or otherwise
affecting warning and informational messages like --silent does.

Note that this is the negated option name documented. You can thus use
--progress-meter to enable the progress meter again.
