Long: include
Short: i
Help: Include protocol response headers in the output
See-also: verbose
---
Include the HTTP response headers in the output. The HTTP response headers can
include things like server name, cookies, date of the document, HTTP version
and more...

To view the request headers, consider the --verbose option.
