Long: netrc-optional
Help: Use either .netrc or URL
Mutexed: netrc
See-also: netrc-file
---
Very similar to --netrc, but this option makes the .netrc usage \fBoptional\fP
and not mandatory as the --netrc option does.
